package de.unisaar.jfap.ex1;

/**
 * Common superclass of {@link Professor} and {@link Student}.
 *
 * @author Jörg Steffen, DFKI
 */
public abstract class Person {

  private String firstName;
  private String lastName;

  /**
   * Create a new person. Only to be used via {@code super()} from subclasses.
   *
   * @param firstName the first name, not {@code null}
   * @param lastName the last name, not {@code null}
   */
  public Person(String firstName, String lastName) {
    checkParamsForNonNull(firstName, lastName);
    this.firstName = firstName;
    this.lastName = lastName;
  }

  /**
   * Convenience method to check if any of the given parameters is {@code null} in which case an
   * {@link IllegalArgumentException} is thrown.
   *
   * @param params the parameters to check
   */
  void checkParamsForNonNull(Object... params) {
    for (Object oneParam : params) {
      if (oneParam == null) {
        throw new IllegalArgumentException("only non-null arguments allowed");
      }
    }
  }

  @Override
  public String toString() {
    return String.format("%s %s", this.firstName, this.lastName).strip();
  }
}
