package de.unisaar.jfap.ex1;

import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatExceptionOfType;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

class ProfessorTest {

  Professor prof;
  Student stud;

  @BeforeEach
  void init() {
    prof = new Professor("Dietrich", "Klakow");
    stud = new Student("Peter", "Mustermann");
  }

  @Test
  void testConstructor() {
    assertThat(prof.toString()).isEqualTo("Prof. Dietrich Klakow");
    assertThat(prof.getStudents()).isEmpty();
  }

  @Test
  void testConstructorWithNullArguments() {

    String exceptionMessage = "only non-null arguments allowed";

    assertThatExceptionOfType(IllegalArgumentException.class)
        .isThrownBy(() -> new Professor(null, null))
        .withMessage(exceptionMessage);

    assertThatExceptionOfType(IllegalArgumentException.class)
        .isThrownBy(() -> new Professor("Dietrich", null))
        .withMessage(exceptionMessage);

    assertThatExceptionOfType(IllegalArgumentException.class)
        .isThrownBy(() -> new Professor(null, "Klakow"))
        .withMessage(exceptionMessage);
  }

  @Test
  void testStudentHandling() {
    prof.addStudent(stud);
    assertThat(prof.getStudents()).containsExactly(stud);
    assertThat(stud.getTeachers()).containsExactly(prof);
    assertThat(prof.removeStudent(stud)).isTrue();
    assertThat(prof.getStudents()).isEmpty();
    assertThat(stud.getTeachers()).isEmpty();
  }

  @Test
  void testAddStudentTwice() {
    prof.addStudent(stud);
    prof.addStudent(stud);
    assertThat(prof.getStudents()).containsExactly(stud);
  }

  @Test
  void testRemoveStudentTwice() {
    prof.addStudent(stud);
    assertThat(prof.removeStudent(stud)).isTrue();
    assertThat(prof.removeStudent(stud)).isFalse();
    assertThat(prof.getStudents()).isEmpty();
  }
}
