package de.unisaar.jfap.ex1;

import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatExceptionOfType;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

class StudentTest {

  Professor prof;
  Student stud;

  @BeforeEach
  void init() {
    prof = new Professor("Dietrich", "Klakow");
    stud = new Student("Peter", "Mustermann");
  }

  @Test
  void testConstructor() {
    assertThat(stud.toString()).isEqualTo("Peter Mustermann");
    assertThat(stud.getTeachers()).isEmpty();
  }

  @Test
  void testConstructorWithNullArguments() {

    String exceptionMessage = "only non-null arguments allowed";

    assertThatExceptionOfType(IllegalArgumentException.class)
        .isThrownBy(() -> new Student(null, null))
        .withMessage(exceptionMessage);

    assertThatExceptionOfType(IllegalArgumentException.class)
        .isThrownBy(() -> new Student("Peter", null))
        .withMessage(exceptionMessage);

    assertThatExceptionOfType(IllegalArgumentException.class)
        .isThrownBy(() -> new Student(null, "Mustermann"))
        .withMessage(exceptionMessage);
  }

  @Test
  void testProfessorHandling() {
    stud.addProfessor(prof);
    assertThat(stud.getTeachers()).containsExactly(prof);
    assertThat(prof.getStudents()).containsExactly(stud);
    assertThat(stud.removeProfessor(prof)).isTrue();
    assertThat(stud.getTeachers()).isEmpty();
    assertThat(prof.getStudents()).isEmpty();
  }

  @Test
  void testAddProfessorTwice() {
    stud.addProfessor(prof);
    stud.addProfessor(prof);
    assertThat(stud.getTeachers()).containsExactly(prof);
  }

  @Test
  void testRemoveProfessorTwice() {
    stud.addProfessor(prof);
    assertThat(stud.removeProfessor(prof)).isTrue();
    assertThat(stud.removeProfessor(prof)).isFalse();
    assertThat(stud.getTeachers()).isEmpty();
  }
}
